/*
FTS: Spectrométrie par TF

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU.

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


new FTS();

function FTS() {
    this.applet = new SimuApplet(this);
    this.applet.appletDoc('fts_doc.xml');
    
	this.dossier = 'donnees';
}

FTS.prototype.calcul = function(lin) {
    var affichages, graphe1, graphe2, out, fspectre, s1, s2, dm, calculerN, n, sc, borne1, borne2,
        cheminf, intensites, s, fs, ddm, i, res, ss, ff, ns, fff, intrf, spectf,
        min, med_spectf, med_ff, facteur, icoupe, max;
    
    affichages = this.applet.getAffichages();
    graphe1 = affichages[0].sousAff[0]; // Affichage
    graphe2 = affichages[0].sousAff[1];
    
    out = new ListeValeurs();
    fspectre = lin.lireString('fspectre');
    s1 = lin.lireDouble('s1');
    s2 = lin.lireDouble('s2');
    dm = lin.lireDouble('D');
    calculerN = lin.lireString('calculerN');
    if (calculerN == 'on')
        n = this.trouverN(s1, s2, dm);
    else
        n = lin.lireEntier('N');
    sc = (n - 1) / (2 * dm);
    borne1 = Math.floor(s1 / sc) * sc;
    borne2 = borne1 + sc;
    out.ajouterString('info', "nb de points: " + n +
        "  borne1: " + borne1 + "  borne2: " + borne2);
    
    // lecture du fichier
    if (this.dossier != null)
        cheminf = this.dossier + '/' + fspectre;
    else
        cheminf = fspectre;
    intensites = this.lireFichier(cheminf);
    if (intensites == null)
        return(null);
    s = intensites[0];
    fs = intensites[1];
    
    // création d'un tableau des différences de marche
    // calculé à partir de D et N (N points, ddm entre 0 et D)
    ddm = new Array(n);
    for (i=0; i<n; i++)
        ddm[i] = dm * i / (n - 1);
    
    // filtrage entre s1 et s2
    // -> résultat = ss et ff
    res = this.simufiltre(s, fs, s1, s2);
    if (res == null)
        return(null);
    ss = res[0];
    ff = res[1];
    ns = ss.length;
    
    // TF
    fff = new Array(ns);
    intrf = new Array(n);
    for (i=0; i<ns; i++)
        fff[i] = ff[i];
    for (i=0; i<n; i++)
        intrf[i] = this.interf_spec(ss, fff, ddm[i]);
    
    // TF
    spectf = new Array(ns);
    for (i=0; i<ns; i++)
        spectf[i] = this.interf_spec(ss[i], intrf, ddm);
    
    // moyenne
    min = spectf[0] / n;
    for (i=0; i<ns; i++) {
        spectf[i] = spectf[i]/n;
        if (spectf[i] < min)
            min = spectf[i];
    }
    // solution peu elegante mais efficace pour la moyenne
    for (i=0; i<ns; i++)
        spectf[i] -= min;
    med_spectf = this.median(spectf);
    med_ff = this.median(ff);
    facteur = med_ff / med_spectf;
    for (i=0; i<ns; i++)
        spectf[i] = spectf[i] * facteur;
    
    // sortie
    graphe1.xdebut = '0';
    graphe1.xfin = '' + dm;
    //graphe1.ydebut = "" + min(intrf);
    //graphe1.yfin = "" + max(intrf);
    //le début est coupé
    icoupe = Math.floor(intrf.length / 6);
    min = intrf[icoupe];
    max = intrf[icoupe];
    for (i=icoupe+1; i<intrf.length; i++) {
        if (intrf[i] < min)
            min = intrf[i];
        if (intrf[i] > max)
            max = intrf[i];
    }
    graphe1.ydebut = '' + min;
    graphe1.yfin = '' + max;
    graphe2.xdebut = '' + this.min(ss);
    graphe2.xfin = '' + this.max(ss);
    graphe2.ydebut = '' + this.min(ff);
    graphe2.yfin = '' + this.max(spectf);
    
    for (i=icoupe; i<n; i++)
        out.ajouterDouble('ddm', ddm[i]);
    for (i=icoupe; i<n; i++)
        out.ajouterDouble('amplitude', intrf[i]);
    for (i=0; i<ns; i++)
        out.ajouterDouble('s0_1', ss[i]);
    for (i=0; i<ns; i++)
        out.ajouterDouble('densite_1', ff[i]);
    for (i=0; i<ns; i++)
        out.ajouterDouble('s0_2', ss[i]);
    for (i=0; i<ns; i++)
        out.ajouterDouble('densite_2', spectf[i]);
    return out;
}

FTS.prototype.trim = function(s) {
    return(s.replace(/^\s+|\s+$/g, ''));
}

/**
 * lire les données d'un fichier ASCII venant d'IDL,
 * avec le nombre d'éléments sur la première ligne
 * les bornes  et l'incrément inc sur la deuxième ligne
 * la position générale du spectre (ignorée) sur la troisième ligne
 * et "y" normalisé sur les suivantes
 * -> t[0][i] = bornemin + increment*i;
 *    t[1][i] = y;
 *    t[0][t[0].length-1] = bornemax; (bornemin + inc*n risque d'être trop grand)
 */
FTS.prototype.lireFichier = function(nomFichier) {
    var xmlhttp, lignes, ligne, t, n, is, sx, sy, bornemax, increment, y, i;
    
    xmlhttp = new XMLHttpRequest();
    if (xmlhttp.overrideMimeType)
        xmlhttp.overrideMimeType('text/plain');
    
    try {
        xmlhttp.open("GET", nomFichier, false);
        xmlhttp.send();
        lignes = xmlhttp.responseText.split("\n");
        
        ligne = lignes[0];
        n = parseInt(this.trim(ligne));
        t = new Array(2);
        for (i=0; i<2; i++)
            t[i] = new Array(n);
        
        ligne = lignes[1];
        ligne = this.trim(ligne);
        is = ligne.indexOf(' ');
        sx = this.trim(ligne.substring(0,is));
        bornemin = parseFloat(sx);
        ligne = this.trim(ligne.substring(is+1));
        is = ligne.indexOf(' ');
        sy = this.trim(ligne.substring(0,is));
        bornemax = parseFloat(sy);
        ligne = this.trim(ligne.substring(is+1));
        increment = parseFloat(ligne);
        
        for (i=0; i<n && ligne != null; i++) {
            ligne = lignes[i + 3];
            if (ligne != null && ligne != '') {
                ligne = this.trim(ligne);
                y = parseFloat(ligne);
                t[0][i] = bornemin + increment*i;
                t[1][i] = y;
            }
        }
        // pour éviter les erreurs d'arrondi
        t[0][t[0].length-1] = bornemax;
        return(t);
    } catch (ex) {
        alert("Erreur à la lecture du fichier de données: " + ex);
        return(null);
    }
}

/**
 * simule un filtre, de bande passante [s1, s2]
 */
//public static Vector simufiltre(double[] s, double[] f, double s1, double s2) {
FTS.prototype.simufiltre = function(s, f, s1, s2) {
    var ind, n, ss, ff, ds, fil, i1, i2, i, res;
    
    ind = this.where(s, s1, s2);
    if (ind == null) {
        alert("simufiltre: Ca coince (s1="+s1+" s2="+s2+")");
        return(null);
    }
    n = ind[1] - ind[0] + 1;
    ss = new Array(n);
    ff = new Array(n);
    for (i=0; i<n; i++) {
        ss[i] = s[i+ind[0]];
        ff[i] = f[i+ind[0]];
    }
    
    // epaisseur des fronts montant et descendant
    ds = (s2 - s1) / 4.5;
    
    // filtre = 1 partout, sauf aux bords
    fil = new Array(n);
    for (i=0; i<n; i++)
        fil[i] = 1;
    i1 = 0;
    i2 = n-1;
    for (i=0; i<n; i++) {
        if (ss[i] < s1 + ds)
            i1 = i;
        if (i2 == n-1 && ss[i] > s2 - ds)
            i2 = i;
    }
    for (i=0; i<=i1; i++)
        fil[i] = (1 + Math.cos(Math.PI * (ss[i] - s1)/ds - Math.PI) ) / 2;
    for (i=i2; i<n; i++)
        fil[i] = (1 + Math.cos(Math.PI * (s2 - ss[i])/ds - Math.PI) ) / 2;
    
    for (i=0; i<n; i++)
        ff[i] = ff[i] * fil[i];
    
    res = new Array();
    res.push(ss);
    res.push(ff);
    return(res);
}

//public static double interf_spec(double s, double[] f, double[] d) {
//public static double interf_spec(double[] s, double[] f, double d) {
FTS.prototype.interf_spec = function(s, f, d) {
    var n, phi, i, som;
    
    if (typeof s == 'number') {
        n = f.length;
        phi = new Array(n);
        for (i=0; i<n; i++)
            phi[i] = 2 * Math.PI * s * d[i];
        som = 0;
        for (i=0; i<n; i++)
            som += f[i] * Math.cos(phi[i]);
        return(som);
    } else {
        n = s.length;
        phi = new Array(n);
        for (i=0; i<n; i++)
            phi[i] = 2 * Math.PI * s[i] * d;
        som = 0;
        for (i=0; i<n; i++)
            som += f[i] * Math.cos(phi[i]);
        return(som);
    }
}

//public static double median(double[] t) {
FTS.prototype.median = function(t) {
    var tc, i;
    tc = new Array(t.length);
    for (i=1; i<t.length; i++)
        tc[i] = t[i];
    this.quicksort(tc);
    return(tc[Math.floor(t.length/2)]);
}

//public static double min(double[] t) {
FTS.prototype.min = function(t) {
    var min, i;
    min = t[0];
    for (i=1; i<t.length; i++)
        if (t[i] < min)
            min = t[i];
    return(min);
}

//public static double max(double[] t) {
FTS.prototype.max = function(t) {
    var max, i;
    max = t[0];
    for (i=1; i<t.length; i++)
        if (t[i] > max)
            max = t[i];
    return(max);
}

//public static void quicksort(double[] t){
//private static void quicksort(double[] t, int lower, int upper){
FTS.prototype.quicksort = function(t, lower, upper) {
    var i, m, temp, pivot;
    
    if (lower == undefined) {
        lower = 0;
        upper = t.length - 1;
    }
    if (lower < upper) {
        temp =  t[lower];
        t[lower] = t[Math.floor((upper+lower)/2)];
        t[Math.floor((upper+lower)/2)] = temp;
        pivot = t[lower];
        m = lower;
        for (i = lower + 1; i <= upper; i++)
            if (t[i] < pivot) {
                m++;
                temp = t[m];
                t[m] = t[i];
                t[i] = temp;
            }
        temp = t[m];
        t[m] = t[lower];
        t[lower] = temp;
        this.quicksort(t, lower, m - 1);
        this.quicksort(t, m + 1, upper);
    }
}

//public int trouverN(double s1, double s2, double dm) {
FTS.prototype.trouverN = function(s1, s2, dm) {
    // CHOIX DU PARAMETRE OPTIMAL
    // algorithme : calcul de la plus petite valeur de N possible, 
    // et iteration jusqu'a trouver le bon intervalle spectral libre
    // entourant les valeurs
    
    var ds, b1, b2, nop, sc, n1;
    
    ds = s2 - s1;  // largeur du spectre
    b1 = 0.;
    b2 = 0.;
    nop = Math.floor(2 * dm * ds); // plus petit entier possible-1
    sc = 0.;
    while (b2 < s2) {
        // on augmente N d'une unite
        nop = nop + 1;
        sc  = nop / (2.*dm);
        // on calcule les bornes inf et sup de l'intervalle
        n1 = Math.floor(s1 / sc);
        b1 = n1 * sc;
        b2 = b1 + sc;
        // si b2 est assez grand, on conclut   
    }
    
    nop = nop + 1; //!!!!!!!!!!!!!!!!!!!!!!!!!!
    return(nop);
}

/**
 * intervalle où t est entre min et max (inclus)
 * on suppose t trié par ordre croissant
 */
//static public int[] where(double[] t, double min, double max) {
FTS.prototype.where = function(t, min, max) {
    var imin, imax, i, res;
    
    imin = -1;
    imax = -1;
    for (i=0; i<t.length; i++) {
        if (t[i] <= min)
            imin = i;
        if (t[i] >= max) {
            imax = i;
            break;
        }
    }
    if (imin == -1 || imax == -1)
        return(null);
    res = [imin, imax];
    return(res);
}
